import { init } from 'echarts-comp/core';
import React, { memo, useMemo, useRef, useState, useLayoutEffect, useEffect, useCallback } from 'react';

const ECharts = memo(function ECharts(props) {
    const resizeDeps = props.resizeDep ?? [];
    const { chartElement, echarts, resize } = useECharts(props);
    useMemo(resize, resizeDeps);
    return chartElement;
});
/** When the theme or initOption changes (shallow comparison), reinitialize Echarts */
function useECharts(config = {}) {
    const { fixedSize, loading, onChange, option, init: init$1, theme, style } = config;
    const domRef = useRef(null);
    const chartRef = useRef();
    const [echarts, setEcharts] = useState();
    const needInit = useEchartsNeedInit(init$1);
    const oldOption = useOldValue(config.option);
    //update options
    useLayoutEffect(() => {
        if (!chartRef.current || objectIsEqual(oldOption, option))
            return;
        updateEchartsOption(chartRef.current, option ?? {});
    });
    //update loading
    useLayoutEffect(refreshLoading, [loading]);
    useLayoutEffect(() => {
        if (!domRef.current) {
            console.error("Unable to get DOM instance");
        }
        const oldInstance = chartRef.current;
        oldInstance?.dispose();
        const instance = init(domRef.current, theme, init$1);
        chartRef.current = instance;
        option && updateEchartsOption(instance, option);
        refreshLoading();
        setEcharts(instance);
        onChange?.(instance, oldInstance);
        return () => instance.dispose();
    }, [theme, needInit]);
    //listen resize
    useEffect(() => {
        if (!fixedSize) {
            const onResize = () => {
                if (timeRef.current !== undefined)
                    return;
                timeRef.current = setTimeout(() => {
                    timeRef.current = undefined;
                    resize();
                }, 40);
            };
            window.addEventListener("resize", onResize);
            return () => {
                //Maybe "Instance xxx has been disposed"
                if (timeRef.current !== undefined)
                    clearTimeout(timeRef.current);
                window.removeEventListener("resize", onResize);
            };
        }
    }, [fixedSize]);
    const timeRef = useRef();
    function refreshLoading() {
        const chart = chartRef.current;
        if (!chart)
            return;
        if (loading)
            chart.showLoading();
        else
            chart.hideLoading();
    }
    const resize = useCallback(() => {
        const echarts = chartRef.current;
        if (!echarts)
            return;
        const parent = domRef.current?.parentElement;
        if (parent) {
            echarts.resize({ width: parent.clientWidth, height: parent.clientHeight });
        }
        else
            echarts.resize();
    }, []);
    const chartElement = (React.createElement("div", { style: { height: "100%", ...style, overflow: "hidden" } },
        React.createElement("div", { ref: domRef, style: { height: "100%" } })));
    return { chartElement, echarts, resize };
}
function updateEchartsOption(instance, opts) {
    instance.setOption(opts ?? {}, true, false);
}
function useOldValue(value) {
    const ref = useRef();
    const oldValue = ref.current;
    ref.current = value;
    return oldValue;
}
function useEchartsNeedInit(initOpts) {
    const oldOpts = useOldValue(initOpts);
    const changeObj = useRef({});
    if (!objectIsEqual(oldOpts, initOpts)) {
        changeObj.current = {};
    }
    return changeObj.current;
}
function objectIsEqual(obj1, obj2) {
    if (obj1 === obj2)
        return true;
    if (typeof obj1 !== "object" || typeof obj2 !== "object")
        return obj1 === obj2;
    if (obj1 === null)
        return obj1 === obj2;
    const keys1 = Object.keys(obj1);
    const keys2 = Object.keys(obj2);
    if (keys1.length !== keys2.length)
        return false;
    for (let i = 0; i < keys2.length; i++) {
        if (obj1[keys2[i]] !== obj2[keys2[i]])
            return false;
    }
    return true;
}

export { ECharts, useECharts };
